/**
 * Copyright  Sergey Melnik (Stanford University, Database Group)
 *
 * All Rights Reserved.
 */

package org.w3c.rdf.model;

import java.util.Enumeration;

/**
 * This is a programming interface to an RDF model.
 * An RDF model is a directed labeled graph.
 * This interface represents a model as container of triples.
 * RDF models must implement <code>getURI()</code> according to a standard MD5-based algorithm.
 *
 * @see org.w3c.tools.crypt.DigestUtil
 * @see org.w3c.rdf.util.RDFDigestUtil
 *
 * @author Sergey Melnik <melnik@db.stanford.edu>
 */

public interface Model extends Resource {

  // General

  /**
   * Set a base URI for the model.
   * Affects creating of new resources and serialization syntax.
   * Inherited method getURI returns the URI set in this method
   */
  public void setSourceURI(String uri);

  /**
   * Returns current base URI setting.
   */
  public String getSourceURI();

  // Model access

  /**
   * Number of triples in the model
   *
   * @return  number of triples, -1 if unknown
   * 
   * @seeAlso org.w3c.rdf.model.VirtualModel
   */
  public int size();

  /**
   * Enumerate triples
   */
  public Enumeration elements();

  /**
   * Tests if the model contains the given triple.
   *
   * @return  <code>true</code> if the triple belongs to the model;
   *          <code>false</code> otherwise.
   */
  public boolean contains(Statement t);

  // Model manipulation: add, remove, find

  /**
   * Adds a new triple to the model.
   */
  public void add(Statement t);

  /**
   * Removes the triple from the model.
   */
  public void remove(Statement t);

  /**
   * General method to search for triples.
   * <code>null</code> input for any parameter will match anything.
   * <p>Example: <code>Model result = m.find( null, RDF.type, new ResourceImpl("http://...#MyClass") )</code>
   * <p>finds all instances of the class <code>MyClass</code>
   */
  public Model find( Resource subject, Resource predicate, RDFNode object );

  /**
   * Search for a single triple.
   * <code>null</code> input for any parameter will match anything.
   *
   * @return  a single triple if such was found;<br>
   *          <code>null</code> if nothing was found;<br>
   *          otherwise a RuntimeException is thrown.
  public Statement find1( Resource subject, Resource predicate, RDFnode object );
   */

  /**
   * Clone the model.
   */
  public Model duplicate();

  /**
   * Creates empty model of the same Class
   */
  public Model create();

  /**
   * Returns the node factory for this model
   */
  public NodeFactory getNodeFactory();

  /**
   * Creates a unique unnamed resource
   */
  //  public Resource noname();
}

